@@ -103,5 +103,5 @@ def get_meadowrun_ssh_key(region_name: str) -> paramiko.PKey:
 
 def download_ssh_key(output_path: str, region_name: str) -> None:
     private_key_text = _get_meadowrun_ssh_key_text(region_name)
-    with open(output_path, "w") as output_file:
+    with open(output_path, "w", encoding="utf-8") as output_file:
         output_file.write(private_key_text)
