@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import abc
+import asyncio
 import dataclasses
 import uuid
 from types import TracebackType
@@ -161,6 +162,10 @@ class InstanceRegistrar(abc.ABC, Generic[_TInstanceState]):
         """
         pass
 
+    @abc.abstractmethod
+    async def authorize_current_ip(self) -> None:
+        pass
+
 
 @dataclasses.dataclass
 class _InstanceWithProposedJobs:
@@ -383,6 +388,10 @@ async def allocate_jobs_to_instances(
     Returns {public_address: [job_ids]}
     """
 
+    authorize_current_ip_task = asyncio.create_task(
+        instance_registrar.authorize_current_ip()
+    )
+
     # TODO this should take interruption_probability_threshold into account for existing
     # instances as well
     allocated = await _choose_existing_instances(
@@ -409,4 +418,6 @@ async def allocate_jobs_to_instances(
             )
         )
 
+    await authorize_current_ip_task
+
     return allocated
